@@ -20,7 +20,7 @@
  * GSSAPI module activation code
  *
  * $RCSfile: gridmap.c,v $
- * $Revision: 1.25.4.2 $
+ * $Revision: 1.25.4.3 $
  * $Date $
  */
 #endif
@@ -197,6 +197,7 @@ globus_gss_assist_gridmap(
                 result,
                 GLOBUS_GSI_GSS_ASSIST_ERROR_WITH_GRIDMAP,
                 (_GASL("Invalid (NULL) user id values")));
+            globus_i_gss_assist_gridmap_line_free(gline);
             goto exit;
 	}
 
@@ -767,6 +768,7 @@ globus_i_gss_assist_gridmap_find_local_user(
         
         if (fgets(line, line_len, gmap_stream) == NULL)
         {
+            free(line);
 	    break;		/* EOF or error */
         }
 
@@ -847,7 +849,7 @@ globus_i_gss_assist_gridmap_find_local_user(
 
     GLOBUS_I_GSI_GSS_ASSIST_DEBUG_EXIT;
     return result;
-} 
+}
 /* gridmap_find_local_user() */
 /* @} */
 
@@ -1408,7 +1410,7 @@ globus_gss_assist_lookup_all_globusid(
     int                                         max_ndx = 512;
     int                                         ndx = 0;
     char **                                     l_dns;
-    globus_i_gss_assist_gridmap_line_t *        gline;
+    globus_i_gss_assist_gridmap_line_t *        gline = NULL;
     char *                                      gridmap_filename = NULL;
     globus_result_t                             res = GLOBUS_SUCCESS;
     FILE *                                      gmap_stream = NULL;
@@ -1480,7 +1482,7 @@ globus_gss_assist_lookup_all_globusid(
         }
 
         if (fgets(line, line_len, gmap_stream) == NULL)
-    {
+        {
             free(line);
             break;
         }
@@ -1507,6 +1509,11 @@ globus_gss_assist_lookup_all_globusid(
                 }
             }
         }
+        if (gline != NULL)
+        {
+            globus_i_gss_assist_gridmap_line_free(gline);
+            gline = NULL;
+        }
         free(line);
     }
     l_dns[ndx] = NULL;
@@ -1841,8 +1848,7 @@ globus_l_gss_assist_gridmap_lookup(
                 result,
                 GLOBUS_GSI_GSS_ASSIST_GRIDMAP_LOOKUP_FAILED,
                 (_GASL("Could not map %s\n"), peer_name_buffer.value));
-            gss_release_buffer(&minor_status, &peer_name_buffer);
-            goto error;
+            goto release_peer_name_buffer;
         }
 
         if(strlen(local_identity) + 1 > identity_buffer_length)
@@ -1871,7 +1877,7 @@ globus_l_gss_assist_gridmap_lookup(
                 (_GASL("Could not map %s to %s\n"),
                  peer_name_buffer.value,
                  desired_identity));
-            goto error;
+            goto release_peer_name_buffer;
         }
 
         if(strlen(desired_identity) + 1 > identity_buffer_length)
@@ -1888,6 +1894,7 @@ globus_l_gss_assist_gridmap_lookup(
         }
     }
 
+release_peer_name_buffer:
     gss_release_buffer(&minor_status, &peer_name_buffer);
 
  error:
@@ -1920,7 +1927,7 @@ globus_l_gss_assist_line_length(
 {
     globus_result_t                     result = GLOBUS_SUCCESS;
     fpos_t                              pos;
-    int                                 line_len;
+    int                                 line_len = -1;
     int                                 rc;
     static char *                       _function_name_ =
         "globus_l_gss_assist_line_length";
@@ -1938,6 +1945,12 @@ globus_l_gss_assist_line_length(
     }
 
     rc = fscanf(fp, "%*[^\n]%*1[\n]%n", &line_len);
+    if (line_len == -1 && rc == 0)
+    {
+       /* match failure; see if we have an empty line */
+       rc = fscanf(fp, "%*1[\n]%n", &line_len);
+    }
+
     if (rc < 0 && !feof(fp))
     {
         GLOBUS_GSI_GSS_ASSIST_ERROR_RESULT(
